@@ -1,5 +1,6 @@
 {-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE LambdaCase #-}
 
 {-
 Copyright (C) 2014 Jesse Rosenthal <jrosenthal@jhu.edu>
@@ -34,7 +35,7 @@ Types and functions for conversion of OMML into TeXMath 'Exp's.
 module Text.TeXMath.Readers.OMML (readOMML) where
 
 import Text.XML.Light
-import Data.Maybe (isJust, mapMaybe, fromMaybe)
+import Data.Maybe (isJust, mapMaybe, fromMaybe, maybeToList)
 import Data.List (intercalate)
 import Data.Char (isDigit, readLitChar)
 import qualified Data.Text as T
@@ -42,6 +43,7 @@ import Text.TeXMath.Types
 import Text.TeXMath.Shared (fixTree, getSpaceWidth, getOperator)
 import Text.TeXMath.Unicode.ToTeX (getSymbolType)
 import Text.TeXMath.Unicode.Fonts (getUnicode, textToFont)
+import Data.List.Split (splitWhen)
 
 readOMML :: T.Text -> Either T.Text [Exp]
 readOMML s | Just e <- parseXMLDoc s =
@@ -57,9 +59,11 @@ unGroup exps = exps
 elemToOMML :: Element -> Maybe [Exp]
 elemToOMML element  | isElem "m" "oMathPara" element = do
   let expList = mapMaybe elemToOMML (elChildren element)
-  return $ map (\l -> if length l == 1 then (head l) else EGrouped l) expList
+  return $ map (\case
+                   [x] -> x
+                   xs -> EGrouped xs) expList
 elemToOMML element  | isElem "m" "oMath" element =
-  Just $ concat $ mapMaybe elemToExps $ unwrapWTags $ elChildren element
+  Just $ elemsToExps $ unwrapWTags $ elChildren element
 elemToOMML _ = Nothing
 
 -- oMath can contain w:hyperlink, w:sdt, etc. I can't find a complete
@@ -113,29 +117,12 @@ data OMathTextStyle = OPlain
                     | OBoldItalic
                     deriving (Show, Eq)
 
-elemToBase :: Element -> Maybe Exp
-elemToBase element | isElem "m" "e" element = do
-  bs <- elemToBases element
-  return $ case bs of
-    (e : []) -> e
-    exps     -> EGrouped exps
-elemToBase _ = Nothing
 
-elemToBases :: Element -> Maybe [Exp]
-elemToBases element | isElem "m" "e" element =
-  return $ concat $ mapMaybe elemToExps' (elChildren element)
-elemToBases _ = Nothing
-
-
--- TODO: The right way to do this is to use the ampersand to break the
--- text lines into multiple columns. That's tricky, though, and this
--- will get us most of the way for the time being.
-filterAmpersand :: Exp -> Exp
-filterAmpersand (EIdentifier s)   = EIdentifier (T.filter ('&' /=) s)
-filterAmpersand (EText tt s)      = EText tt (T.filter ('&' /=) s)
-filterAmpersand (EStyled tt exps) = EStyled tt (map filterAmpersand exps)
-filterAmpersand (EGrouped exps)   = EGrouped (map filterAmpersand exps)
-filterAmpersand e                    = e
+breakOnAmpersand :: [Exp] -> [[Exp]]
+breakOnAmpersand = splitWhen isAmpersand
+ where
+  isAmpersand (ESymbol _ "&") = True
+  isAmpersand _ = False
 
 elemToOMathRunTextStyle :: Element -> OMathRunTextStyle
 elemToOMathRunTextStyle element
@@ -234,18 +221,42 @@ oMathRunTextStyleToTextType (Styled scr sty)
     Just $ TextBoldItalic
   | otherwise = Nothing
 
+-- merge adjacent Exps with same style; see jgm/pandoc#10560:
+mergeExps :: [Exp] -> [Exp]
+mergeExps [] = []
+mergeExps (EStyled tt xs : EStyled tt' xs' : rest)
+  | tt == tt' = mergeExps (EStyled tt (xs <> xs') : rest)
+mergeExps (EText tt x : EText tt' x' : rest)
+  | tt == tt' = mergeExps (EText tt (x <> x') : rest)
+mergeExps (x:xs) = x : mergeExps xs
+
+elemsToExps :: [Element] -> [Exp]
+elemsToExps = mergeExps . concat . mapMaybe elemToExps
+
 elemToExps :: Element -> Maybe [Exp]
 elemToExps element = unGroup <$> elemToExps' element
 
+elemToBase :: Element -> Maybe Exp
+elemToBase element | isElem "m" "e" element = do
+  bs <- elemToBases element
+  return $ case bs of
+    (e : []) -> e
+    exps     -> EGrouped exps
+elemToBase _ = Nothing
+
+elemToBases :: Element -> Maybe [Exp]
+elemToBases element | isElem "m" "e" element =
+  return $ concat $ mapMaybe elemToExps' (elChildren element)
+elemToBases _ = Nothing
+
+
+
 elemToExps' :: Element -> Maybe [Exp]
 elemToExps' element | isElem "m" "acc" element = do
   let chr = filterChildName (hasElemName "m" "accPr") element >>=
             filterChildName (hasElemName "m" "chr") >>=
-            findAttrBy (hasElemName "m" "val") >>=
-            Just . head
-      chr' = case chr of
-        Just c -> T.singleton c
-        Nothing -> "\x302"       -- default to wide hat.
+            findAttrBy (hasElemName "m" "val")
+      chr' = maybe "\x302" T.pack chr  -- default to wide hat
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
   return $ [EOver False baseExp (ESymbol Accent chr')]
@@ -274,18 +285,17 @@ elemToExps' element | isElem "m" "d" element =
                   (elChildren element)
       inDelimExps = map (map Right) baseExps
       dPr = filterChildName (hasElemName "m" "dPr") element
+      headOrSpace [] = Just ' '
+      headOrSpace (c:_) = Just c
       begChr = dPr >>=
                filterChildName (hasElemName "m" "begChr") >>=
-               findAttrBy (hasElemName "m" "val") >>=
-               (\c -> if null c then (Just ' ') else (Just $ head c))
+               findAttrBy (hasElemName "m" "val") >>= headOrSpace
       sepChr = dPr >>=
                filterChildName (hasElemName "m" "sepChr") >>=
-               findAttrBy (hasElemName "m" "val") >>=
-               (\c -> if null c then (Just ' ') else (Just $ head c))
+               findAttrBy (hasElemName "m" "val") >>= headOrSpace
       endChr = dPr >>=
                filterChildName (hasElemName "m" "endChr") >>=
-               findAttrBy (hasElemName "m" "val") >>=
-               (\c -> if null c then (Just ' ') else (Just $ head c))
+               findAttrBy (hasElemName "m" "val") >>= headOrSpace
       beg = maybe "(" T.singleton begChr
       end = maybe ")" T.singleton endChr
       sep = maybe "|" T.singleton sepChr
@@ -294,17 +304,19 @@ elemToExps' element | isElem "m" "d" element =
    Just [EDelimited beg end exps]
 elemToExps' element | isElem "m" "eqArr" element =
   let expLst = mapMaybe elemToBases (elChildren element)
-      expLst' = map (\es -> [map filterAmpersand es]) expLst
+      expLst' = map breakOnAmpersand expLst
+      cols = maximum (map length expLst')
+      colspecs = take cols $ cycle [AlignRight , AlignLeft]
   in
-   return [EArray [] expLst']
+   return [EArray colspecs expLst']
 elemToExps' element | isElem "m" "f" element = do
   num <- filterChildName (hasElemName "m" "num") element
   den <- filterChildName (hasElemName "m" "den") element
   let barType = filterChildName (hasElemName "m" "fPr") element >>=
             filterChildName (hasElemName "m" "type") >>=
             findAttrBy (hasElemName "m" "val")
-  let numExp = EGrouped $ concat $ mapMaybe (elemToExps) (elChildren num)
-      denExp = EGrouped $ concat $ mapMaybe (elemToExps) (elChildren den)
+  let numExp = EGrouped $ elemsToExps (elChildren num)
+      denExp = EGrouped $ elemsToExps (elChildren den)
   case barType of
     Just "noBar" -> Just [EFraction NoLineFrac numExp denExp]
     _            -> Just [EFraction NormalFrac numExp denExp]
@@ -312,7 +324,7 @@ elemToExps' element | isElem "m" "func" element = do
   fName <- filterChildName (hasElemName "m" "fName") element
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
           elemToBase
-  let fnameExp = case mconcat $ mapMaybe (elemToExps') (elChildren fName) of
+  let fnameExp = case mconcat $ mapMaybe elemToExps' (elChildren fName) of
                    [x] -> x
                    xs  -> EGrouped xs
   return [fnameExp, baseExp]
@@ -352,14 +364,14 @@ elemToExps' element | isElem "m" "limLow" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element
           >>= elemToBase
   limExp <- filterChildName (hasElemName "m" "lim") element
-            >>= (\e -> Just $ concat $ mapMaybe (elemToExps) (elChildren e))
+            >>= (\e -> Just $ elemsToExps (elChildren e))
             >>= (return . EGrouped)
   return [EUnder True baseExp limExp]
 elemToExps' element | isElem "m" "limUpp" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element
           >>= elemToBase
   limExp <- filterChildName (hasElemName "m" "lim") element
-            >>= (\e -> Just $ concat $ mapMaybe (elemToExps) (elChildren e))
+            >>= (\e -> Just $ elemsToExps (elChildren e))
             >>= (return . EGrouped)
   return [EOver True baseExp limExp]
 elemToExps' element | isElem "m" "m" element =
@@ -383,22 +395,22 @@ elemToExps' element | isElem "m" "nary" element = do
                filterChildName (hasElemName "m" "limLoc") >>=
                findAttrBy (hasElemName "m" "val")
   subExps <- filterChildName (hasElemName "m" "sub") element >>=
-         (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+         return . elemsToExps . elChildren
   supExps <- filterChildName (hasElemName "m" "sup") element >>=
-         (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
-  baseExp <- filterChildName (hasElemName "m" "e") element >>=
-             elemToBase
+         return . elemsToExps . elChildren
+  let baseExp = maybeToList $
+        filterChildName (hasElemName "m" "e") element >>= elemToBase
   case limLoc of
-    Just "undOvr" -> return [EUnderover True
+    Just "undOvr" -> return $ EUnderover True
                               (ESymbol Op opChr)
                               (EGrouped subExps)
                               (EGrouped supExps)
-                            , baseExp]
-    _             -> return [ESubsup
+                            : baseExp
+    _             -> return $ ESubsup
                               (ESymbol Op opChr)
                               (EGrouped subExps)
                               (EGrouped supExps)
-                            , baseExp]
+                            : baseExp
 
 elemToExps' element | isElem "m" "phant" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
@@ -406,7 +418,7 @@ elemToExps' element | isElem "m" "phant" element = do
   return [EPhantom baseExp]
 elemToExps' element | isElem "m" "rad" element = do
   degExps <- filterChildName (hasElemName "m" "deg") element >>=
-              (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+              return . elemsToExps . elChildren
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
   return $ case degExps of
@@ -414,9 +426,9 @@ elemToExps' element | isElem "m" "rad" element = do
     ds -> [ERoot (EGrouped ds) baseExp]
 elemToExps' element | isElem "m" "sPre" element = do
   subExps <- filterChildName (hasElemName "m" "sub") element >>=
-            (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+            return . elemsToExps . elChildren
   supExps <- filterChildName (hasElemName "m" "sup") element >>=
-            (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+            return . elemsToExps . elChildren
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
   return [ESubsup
@@ -428,21 +440,21 @@ elemToExps' element | isElem "m" "sSub" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
   subExps <- filterChildName (hasElemName "m" "sub") element >>=
-            (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+            return . elemsToExps . elChildren
   return [ESub baseExp (EGrouped subExps)]
 elemToExps' element | isElem "m" "sSubSup" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
   subExps <- filterChildName (hasElemName "m" "sub") element >>=
-             (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+             return . elemsToExps . elChildren
   supExps <- filterChildName (hasElemName "m" "sup") element >>=
-             (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+             return . elemsToExps . elChildren
   return [ESubsup baseExp (EGrouped subExps) (EGrouped supExps)]
 elemToExps' element | isElem "m" "sSup" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
   supExps <- filterChildName (hasElemName "m" "sup") element >>=
-            (\e -> return $ concat $ mapMaybe (elemToExps) (elChildren e))
+            return . elemsToExps . elChildren
   return [ESuper baseExp (EGrouped supExps)]
 elemToExps' element | isElem "m" "r" element = do
   let mrPr = filterChildName (hasElemName "m" "rPr") element
@@ -486,8 +498,9 @@ getSymChar :: Element -> T.Text
 getSymChar element
   | Just s <- lowerFromPrivate <$> getCodepoint
   , Just font <- getFont =
-  let [(char, _)] = readLitChar ("\\x" ++ s) in
-    maybe "" T.singleton $ getUnicode font char
+  case readLitChar ("\\x" ++ s) of
+     [(char, _)] -> maybe "" T.singleton $ getUnicode font char
+     _ -> ""
   where
     getCodepoint = findAttrBy (hasElemName "w" "char") element
     getFont = (textToFont . T.pack) =<< findAttrBy (hasElemName "w" "font") element
