@@ -26,11 +26,13 @@ where
 
 import Text.XML.Light
 import Text.TeXMath.Types
+import Text.TeXMath.Shared (isUppercaseGreek, isRLSequence)
 import Data.Generics (everywhere, mkT)
 import Data.Char (isSymbol, isPunctuation)
 import Data.Either (lefts, isLeft, rights)
 import qualified Data.Text as T
 import Data.List.Split  (splitWhen)
+import Data.List (intercalate)
 
 -- | Transforms an expression tree to an OMML XML Tree
 writeOMML :: DisplayType -> [Exp] -> Element
@@ -83,12 +85,17 @@ maximum' [] = 0
 maximum' xs = maximum xs
 
 makeArray :: [Element] -> [Alignment] -> [ArrayLine] -> Element
+makeArray props as rs
+  | isRLSequence as = mnode "eqArr" $ map toE rs
+  where toE r = mnode "e" (intercalate ampersand
+                            (map (concatMap (showExp props)) r))
+        ampersand = [mnode "r" [ mnode "t" ("&" :: String) ]]
 makeArray props as rs = mnode "m" $ mProps : map toMr rs
   where mProps = mnode "mPr"
                   [ mnodeA "baseJc" "center" ()
                   , mnodeA "plcHide" "on" ()
                   , mnode "mcs" $ map toMc as' ]
-        as'    = take (maximum' $ map length rs) $ as ++ cycle [AlignCenter]
+        as'    = take (maximum' $ map length rs) $ as ++ repeat AlignCenter
         toMr r = mnode "mr" $ map (mnode "e" . concatMap (showExp props)) r
         toMc a = mnode "mc" $ mnode "mcPr"
                             [ mnodeA "mcJc" (toAlign a) ()
@@ -111,17 +118,17 @@ setProps tt =
        TextNormal       -> [sty "p"]
        TextBold         -> [sty "b"]
        TextItalic       -> [sty "i"]
-       TextMonospace    -> [sty "p", scr "monospace"]
-       TextSansSerif    -> [sty "p", scr "sans-serif"]
-       TextDoubleStruck -> [sty "p", scr "double-struck"]
-       TextScript       -> [sty "p", scr "script"]
-       TextFraktur      -> [sty "p", scr "fraktur"]
+       TextMonospace    -> [scr "monospace", sty "p"]
+       TextSansSerif    -> [scr "sans-serif", sty "p"]
+       TextDoubleStruck -> [scr "double-struck", sty "p"]
+       TextScript       -> [scr "script", sty "p"]
+       TextFraktur      -> [scr "fraktur", sty "p"]
        TextBoldItalic    -> [sty "bi"]
-       TextSansSerifBold -> [sty "b", scr "sans-serif"]
-       TextBoldScript    -> [sty "b", scr "script"]
-       TextBoldFraktur   -> [sty "b", scr "fraktur"]
-       TextSansSerifItalic -> [sty "i", scr "sans-serif"]
-       TextSansSerifBoldItalic -> [sty "bi", scr "sans-serif"]
+       TextSansSerifBold -> [scr "sans-serif", sty "b"]
+       TextBoldScript    -> [scr "script", sty "b"]
+       TextBoldFraktur   -> [scr "fraktur", sty "b"]
+       TextSansSerifItalic -> [scr "sans-serif", sty "i"]
+       TextSansSerifBoldItalic -> [scr "sans-serif", sty "bi"]
    where sty x = mnodeA "sty" x ()
          scr x = mnodeA "scr" x ()
 
@@ -223,7 +230,10 @@ showExp props e =
 
    EIdentifier ""   -> [str props "\x200B"]  -- 0-width space
                        -- to avoid the dashed box we get otherwise; see #118
-   EIdentifier x    -> [str props x]
+   EIdentifier x
+     | isUppercaseGreek x
+     , null props -> [str [mnodeA "sty" "p" ()] x]
+     | otherwise  -> [str props x]
    EMathOperator x  -> [str (mnodeA "sty" "p" () : props) x]
    ESymbol ty xs
     | Just (c, xs') <- T.uncons xs
