@@ -199,16 +199,18 @@ writeExp (ESub b e1) = do
   tellGroup (writeExp e1)
 writeExp (ESuper b e1) = do
   (if isFancy b then tellGroup else id) $ writeExp b
-  case e1 of
-    ESymbol Ord "\8242" -> tell [Token '\'']
+  case S.toPrimes e1 of
+    Just ps -> tell (replicate (T.length ps) (Token '\''))
     _ -> do tell [Token '^']
             tellGroup (writeExp e1)
 writeExp (ESubsup b e1 e2) = do
   (if isFancy b then tellGroup else id) $ writeExp b
   tell [Token '_']
   tellGroup (writeExp e1)
-  tell [Token '^']
-  tellGroup (writeExp e2)
+  case S.toPrimes e2 of
+    Just ps -> tell (replicate (T.length ps) (Token '\''))
+    _ -> do tell [Token '^']
+            tellGroup (writeExp e2)
 writeExp (EOver convertible b e1) = do
   env <- asks mathEnv
   case xarrow b of
@@ -285,7 +287,7 @@ writeExp (EStyled ttype es) = do
   tell [ControlSeq txtcmd]
   tellGroup (mapM_ writeExp $ everywhere (mkT (fromUnicode ttype)) es)
 writeExp (EArray as rows)
-  | isRLSequence as = do
+  | S.isRLSequence as = do
   env <- asks mathEnv
   if "amsmath" `elem` env
      then table "aligned" [] rows
@@ -296,11 +298,6 @@ writeExp (EArray aligns rows) = do
      then table "matrix" [] rows
      else table "array" aligns rows
 
-isRLSequence :: [Alignment] -> Bool
-isRLSequence [AlignRight, AlignLeft] = True
-isRLSequence (AlignRight : AlignLeft : as) = isRLSequence as
-isRLSequence _ = False
-
 table :: T.Text -> [Alignment] -> [ArrayLine] -> Math ()
 table name aligns rows = do
   tell [ControlSeq "\\begin", Grouped [Literal name]]
